#include "../LLExecCore/PluginsInterface.h"
#include "../LLExecCore/TraceLog.h"
#include "io.h"

#define NUM_INPUTS 8
#define NUM_OUTPUTS 8

#define DB_INPUT_MAX_SIZE 64
#define DB_OUTPUT_MAX_SIZE 64

//	------------------- access to API functions of LLEXECCORE ---------
LLEXEC_API m_plcApi;
static uint8_t m_stateInput;
static uint8_t m_stateOutput;
static bool_t * m_inputs;
static bool_t * m_outputs;
static bool_t m_ioEnabled;


//	------------------- PLC functions export ---------

// this will be called automatically in the system startup sequence
static bool_t initFunction()
{
	if (io_init() == 0)
	{
		m_plcApi.LLPrintLog(ALTRACE_LEV_PLG_BASE, _T("PiFace initialized!"));
		m_ioEnabled = TRUE;
	}
	else
	{
		m_ioEnabled = FALSE;
	}
	
	// get data block pointers
	m_inputs =  (bool_t*) m_plcApi.LLGetDataBlockAddress(DBTY_INP, 0, 0);
	m_outputs =  (bool_t*) m_plcApi.LLGetDataBlockAddress(DBTY_OUT, 0, 0);
	
	if ((m_inputs == NULL) || (m_outputs == NULL))
	{
		m_plcApi.LLPrintLog(ALTRACE_LEV_PLG_BASE, _T("ERROR: failed to get db addresses!"));
		return FALSE; // this stops runtime initialization
	}
	
	return TRUE;
}

// this is called before every I/O cycle, it do effective data-exchange from PLC to PIFACE
static void syncIO()
{
	int idx;
	uint8_t byte;
	
	if (m_ioEnabled)
		m_stateInput = io_read_input();
	
	byte = m_stateInput;
	for (idx = 0; idx < NUM_OUTPUTS; idx++)
	{
		m_inputs[idx] = byte & 0x1;
		byte >>= 1;
	}
	
	byte = 0;
	for (idx = NUM_OUTPUTS -1; idx >= 0; idx--)
	{
		if (m_outputs[idx])
			byte |= 1;
		else
			byte &= ~1;
		if (idx)
			byte <<= 1;			
	}
	
	m_stateOutput = byte;
	if (m_ioEnabled)
		io_write_output(m_stateOutput);
}

// this function is exposed to the PLC runtime,
// it can be called from PLC code
static bool_t PIFACE_resetAllOutputs(int dummy_input)
{
	m_plcApi.LLPrintLog(ALTRACE_LEV_PLG_BASE, _T("Call to PIFACE_resetAllOutputs!"));
	if (!m_ioEnabled)
		return FALSE;
		
	memset(m_outputs, 0, NUM_OUTPUTS * sizeof(bool_t));
	m_plcApi.LLPrintLog(ALTRACE_LEV_PLG_BASE, _T("Reset all PIFACE outputs!"));
	return TRUE;
}

// this is the table of function exposed to the runtime
static LLEXEC_FUNDEF m_Functions[] =
{
	{ LLFUNCT_INIT_L0,	(void *)initFunction, "initFunction", }, // init function
	{ LLFUNCT_IO,	(void *)syncIO,		"PLCsyncIO", }, // io_function: is called before every PLC cycle
	{ LLFUNCT_PLC,	(void *)PIFACE_resetAllOutputs, "PIFACE_resetAllOutputs", }, // plc exported function
};

LLXPLG_GETFUNCTIONDEFS
{
	*num = ARRDIM(m_Functions);
	*defs = m_Functions;
	return TRUE;
}


//	------------------- DataBlocks definition ---------
static LLEXEC_DBDEF m_DataBlocks[] =
{
	{ FALSE, DBTY_INP,	0,	0,	DB_INPUT_MAX_SIZE,		sizeof(bool_t),	DBRW_R, },
	{ FALSE, DBTY_OUT,	0,	0, DB_OUTPUT_MAX_SIZE,		sizeof(bool_t),	DBRW_RW, },
};

LLXPLG_INITIALIZEPLUGIN
{
	m_plcApi = *api;
	return TRUE;
}

LLXPLG_GETDATABLOCKSDEFS
{
	*num = ARRDIM(m_DataBlocks);
	*defs = m_DataBlocks;
	return TRUE;
}

